function create_desc_db(varargin)
% James Hays 5 22 2009
% Frank Palermo 9 17 2012
%
% converts scene descriptors for a database of images.
% images are expected to be stored in a 
%   root_dir / base_dir / sub_dir / [1000 or so images]
% directory structure.
%
% This function will call create_desc_dir (or some variant thereof) on each
% sub directory. It will process the subdirectories in random order and
% check for locks from parallel threads.
%
% the output structure will mirror the input structure.
%
% input argument 1 - directory of input images
% input argument 2 - output directory of image descriptors
% input argument 3 - "Response Curve Mode" (Optional, Boolean)
%                       0/False [Default]: Computes all image features and 
%                           extracts all Flickr-provided metadata.
%                       1/True: Computes only the joint histograms needed for 
%                           response curve estimation, extracts only the image 
%                           year (from the directory structure).
% input argument 4 - "Ignore Metadata" (Optional, Boolean)
%                       0/False [Default]: Extracts and saves all supported
%                           metadata for the given mode.
%                       1/True: Does not extract or save any image metadata.
%
img_path = varargin{1};
desc_path = varargin{2};
if(nargin < 3)
    curve_mode = 0;
else
    curve_mode = varargin{3};
end
if(nargin < 4)
    ignore_meta = 0;
else
    ignore_meta = varargin{4};
end

[status, hostname] = system('hostname');
fprintf('Running create_desc_db() on %s at %s\n', hostname, datestr(now));

addpath('../util')

base_dirs = real_dirs(img_path);
all_sub_dirs = cell(0);

for i = 1:length(base_dirs)
    cur_sub_dirs = real_dirs( fullfile( img_path, base_dirs{i}) );
    cur_sub_dirs = strcat( base_dirs{i}, filesep, cur_sub_dirs ); %single string, single string, many rows of strings
    all_sub_dirs = [all_sub_dirs; cur_sub_dirs];
end

%use_locks determines whether the function will create and check for lock
%files while running. Only matters if multiple processes are running in
%parallel.
use_locks = 1;

num_dirs  = length(all_sub_dirs);

fprintf('Calculating descriptors for each subdirectory, in random order\n');
rand('twister', sum(100*clock)); %seed random number generator
dirs_to_use = randperm(num_dirs);

%%%%%%%%%%%%%
% Main Loop
for dir_idx = dirs_to_use
    current_dir_name = all_sub_dirs{dir_idx};
    lock_file_name = [current_dir_name '.lock'];
    lock_file_name( lock_file_name == filesep ) = '_';
    
    if(use_locks)
        fprintf('Checking for lock on %s\n', current_dir_name);
        %check to see if this block is locked or complete
        
        if( exist(fullfile(desc_path, current_dir_name),'file') || ...
            exist(fullfile(desc_path, lock_file_name), 'file') )
        
            fprintf(' ! dir %s already locked or computed, skipping\n', current_dir_name);
            continue;
        else
            diary( fullfile(desc_path, lock_file_name))
            fprintf(' locking dir %s\n', current_dir_name);
            fprintf('Locking this directory from %s at %s\n', hostname, datestr(now));
            fprintf('cputime: %f\n', cputime * 10e3);
            fprintf('random key: %d %d %d %d %d\n', round(rand(1) * 10e5), round(rand(1) * 10e5), round(rand(1) * 10e5), round(rand(1) * 10e5), round(rand(1) * 10e5))
            [success, message, messageid] = mkdir( fullfile(desc_path, current_dir_name));
            if(strcmp(messageid, 'MATLAB:MKDIR:DirectoryExists'))
                fprintf(' ERROR, processes are completely synched, backing off\n');
                diary off;
                continue;
            end
            diary off;
        end
    end

    create_desc_dir(fullfile(img_path, current_dir_name), ...
    	fullfile(desc_path, current_dir_name), curve_mode, ignore_meta);

    if(use_locks) %locks only indicate works in progress (or crashes)
        delete( fullfile(desc_path, lock_file_name) )
    end
end %end of all sub directories

fprintf('Finished create_desc_db\n');
