function create_desc_dir(varargin)
% James Hays 11/21/2007
% Frank Palermo 9/17/2012
%
% Computes various image features as described in Section 5.1 of:
%	Palermo, F., Hays, J., Efros, A.A.: Dating Historical Color Images. In:
%	Fitzgibbon, A., Lazebnik, S., Sato, Y., Schmid, C. (eds.) ECCV 2012, Part VI.
%	LNCS, vol. 7577, pp. 499-512. Springer, Heidelberg (2012)
% including:
%	"Color Co-occurrence Histogram"
%	"Conditional Probability of Saturation Given Hue"
%	"Hue Histogram"
%	"Gist Descriptor"
%	"Tiny Image"
%	"L*a*b* Color Histogram"
% on a directory of input images.
%
% input argument 1 - directory of input images
% input argument 2 - output directory of image descriptors
% input argument 3 - "Response Curve Mode" (Optional, Boolean)
%						0/False [Default]: Computes all image features and 
%							extracts all Flickr-provided metadata.
%						1/True: Computes only the joint histograms needed for 
%							response curve estimation, extracts only the image 
%							year (from the directory structure).
% input argument 4 - "Ignore Metadata" (Optional, Boolean)
%                       0/False [Default]: Extracts and saves all supported
%                           metadata for the given mode.
%                       1/True: Does not extract or save any image metadata.
%
input_path = varargin{1};
output_path = varargin{2};
if(nargin < 3)
	curve_mode = 0;
else
	curve_mode = varargin{3};
end
if(nargin < 4)
    ignore_meta = 0;
else
    ignore_meta = varargin{4};
end

addpath('calc_gist');
addpath('calc_photo_priors_jh');
addpath('../third_party');

fprintf('\nProcessing images in directory %s\n', input_path);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Main Loop
input_list = dir(input_path);
for i = 1:length(input_list)
	current_filename = input_list(i).name;
	%check for readable file formats
	if (size(strfind(current_filename, '.jpg'),1) > 0)
		current_img_file = fullfile(input_path, current_filename);
			
		fprintf('\nCalculating descriptors for %s\n', current_img_file);
		t0 = clock;
				
		try
			current_img = imread(current_img_file);
		catch
			diary_name = sprintf('read_error_log_%s.txt', current_filename);
			diary(diary_name)
			fprintf('%s : Failed to read file %s\n', datestr(now), ...
				current_img_file);
			lasterr
			diary off
					
			%hacky, continue with invalid image
			current_img = zeros(800,800,3);
		end

		output_filename = strcat(current_filename(1:end-3), 'mat');

		input_path_toks = regexp(input_path, filesep, 'split');
		desc_filename_end = [input_path_toks{end-1} '_' input_path_toks{end} '_' ...
			current_filename(1:end-4)];
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		current_img = im2double(current_img);

		% -- Features --
		[photo_priors_jh] = calc_photo_priors_jh(current_img);

		if ~curve_mode
			% Calculate other image features described in Section 5.1.

			% -- "Color Co-occurrence Histogram" --
			co_occur_hist = imresize(photo_priors_jh, [32 32], 'bilinear', ...
				'Antialiasing', false);
			[co_occur_hist] = co_occur_hist ./ sum(co_occur_hist(:));

			% -- "Gist Descriptor" (Oliva and Torralba, 2001) --
			alt_gist = gist(rgb2gray(current_img), 6, 4, 15, 15);
			alt_gist = single(imresize(alt_gist, [5 5], 'bilinear', ...
				'Antialiasing',false));
			[gist_image] = alt_gist ./ max(alt_gist(:));
			
			% -- "Tiny Image" (Torralba et. al., 2008) --
			[tiny_image] = single(imresize(current_img, [32 32]));

			% (The remaining features are computed in either CIELAB or
			% 	CIE L*C*h(a*b*) and require the removal of pixels which 
			% 	are very close to gray).
			[L, a, b, C, h] = RGB2LAB_and_LCh_with_gray_removal(current_img);

			% -- "Conditional Probability Of Saturation Given Hue" --
			s = C ./ L; % Approximation by analogy from CIELUV
			s_thresh = prctile(s, 99);
			hs_hist = histnd([h s], ...
				linspace(-pi,pi,256), linspace(0,s_thresh,256));
			[h_mean_s, h_std_s] = hist2dist(hs_hist, 0, s_thresh, 256);
			[P_S_Given_H] = ...
				[((h_mean_s ./ (2*pi)) + 0.5) ((h_std_s ./ (2*pi)) + 0.5)];

			% -- "Hue Histogram" --
			hue_hist = histc(h, linspace(-pi,pi,128));
			nan_idx = isnan(hue_hist);
			hue_hist(nan_idx) = 0;
			[hue_hist] = hue_hist ./ sum(hue_hist);

			% -- "L*a*b* Color Histogram" (Hays and Efros, 2008) --
			CIELAB_hist = histnd([L a b], ...
				[-inf 55 70 90 inf], ...
				[-inf -30 -25 -20 -15 -10 -5 0 5 10 15 20 25 30 inf], ...
				[-inf -30 -25 -20 -15 -10 -5 0 5 10 15 20 25 30 inf]);
			CIELAB_hist = CIELAB_hist(1:4, 1:14, 1:14);
			[CIELAB_hist] = CIELAB_hist ./ sum(CIELAB_hist(:));
		end
		% -- End Features --
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

		% Metadata
		if ~ignore_meta
			if ~curve_mode
				% Extract the full set of Flickr metadata.
				fprintf('Extracting metadata\n');
				try
					[file_info] = imfinfo(fullfile(input_path, ...
						current_filename));
					[interestingness, latitude, longitude, date_vec, title, ...
						tags, photographer, license, accuracy] = ...
						parse_metadata2(file_info.Comment);
				catch
					diary_name = sprintf('parse_metadata_error_log_%s.txt', ...
						output_filename);
					diary(diary_name)
					fprintf('%s : parse_metadata crashed on %s\n', datestr(now), ...
						fullfile(output_path, output_filename));
					lasterr
					diary off
					end
			else
				% Otherwise, only a partial date vector (i.e. the year) is 
				%	supported (extracted from the directory structure).
				desc_filename_end_toks = regexp(desc_filename_end, '_', 'split');
				date_vec = [str2num(desc_filename_end_toks{1})];
			end
		end

		% Save Descriptor
		try
			if ~curve_mode
				% The full set of metadata and all features are supported.
				if ~ignore_meta
					save(fullfile(output_path, output_filename), ...
						'photo_priors_jh', ...
						'co_occur_hist', ...
						'gist_image', ...
						'tiny_image', ...
						'P_S_Given_H', ...
						'hue_hist', ...
						'CIELAB_hist', ...
						'interestingness', 'latitude', 'longitude', ...
						'date_vec', 'title', 'tags', 'photographer', ...
						'license', 'accuracy', '-V6');
				else
					save(fullfile(output_path, output_filename), ...
						'photo_priors_jh', ...
						'co_occur_hist', ...
						'gist_image', ...
						'tiny_image', ...
						'P_S_Given_H', ...
						'hue_hist', ...
						'CIELAB_hist', ...
						'-V6');
				end
			else
				if ~ignore_meta
					save(fullfile(output_path, output_filename), ...
						'photo_priors_jh', ...
						'date_vec', '-V6');
				else
					save(fullfile(output_path, output_filename), ...
						'photo_priors_jh', ...
						'-V6');
				end
			end
			
			fprintf('Descriptors computed and saved in %f seconds\n',  etime(clock,t0))
		catch
			diary_name = sprintf('save_error_log_%s.txt', output_filename);
			diary(diary_name)
			fprintf('%s : Failed to save file %s\n', datestr(now), ...
				fullfile(output_path, output_filename));
			fprintf('  Likely causes- file io error, or failure to compute a descriptor\n');
			lasterr
			diary off
		end

		clear photo_priors_jh
		clear co_occur_hist
		clear gist_image
		clear tiny_image
		clear L a b C h
		clear P_S_Given_H
		clear hue_hist
		clear CIELAB_hist

		clear interestingness
		clear latitude
		clear longitude
		clear date_vec
		clear title
		clear tags
		clear photographer
		clear license
		clear accuracy
	end
end
