% downloadphotos_int_ii - Given a directory containing Flickr query result
%	text files (generated by the code in query_imgs), attempts to download
%	all of the images listed in the result files into the specified output 
%	directory. The function attempts to automatically detect and trim 
%	any solid color borders from the images and resaves the trimmed images 
%	as high-quality JPEG files. Certain Flickr metadata from the query 
%	result files is formatted and inserted in the EXIF "Comment" field of 
%	the newly-created JPEGs.
%
%	This function requires GNU Wget (tested with version 1.14). You may 
%	need to edit the variable WGET_PATH depending on the location of your
%	Wget installation.
%
%	This function downloads Flickr "Medium" size images by default, but 
%	can optionally download "Original" images. In this case, ExifTool 
%	(tested with version 9.01) is required in order to preserve the 
%	majority of the image's original EXIF tags in the newly-created JPEG. 
%	You may need to edit the variable EXIFTOOL_PATH depending on the 
%	location of your ExifTool installation.
%	
%	Originally written by Tamara Berg, extended by James Hays,
% 	and modified by Frank Palermo.
%
%	Parameters:
%		search_result_dir - A string containing the full path to a directory
%			of Flickr query result text files.
%		output_dir - A string containing the full path to a directory where
%			the downloaded images will be saved after trimming and metadata
%			insertion.
%		osize - A boolean flag which determines whether "Medium" or 
%			"Original" size Flickr images will be downloaded.
%				0/False [Default]: Download "Medium" (500px on the longest 
%					dimension) images.
%				1/True: Download "Original" images, but only if they are at 
%					least of "Large" (>1024px on the longest dimension) size.
%					Otherwise, ignore them and proceed to the next image.
%
function downloadphotos_int_ii(search_result_dir, output_dir, osize)
% -- Settings --
WGET_PATH = '/usr/local/bin/wget';
EXIFTOOL_PATH = '/usr/local/bin/exiftool';
% -- End Settings --

% 'Temp' Directory
if(ispc())
	% Windows
	TEMP_DIR = fullfile('C:','Temp');
else
	% Unix, Mac OS X
	TEMP_DIR = '/tmp/';
end

% Image Size & Aspect Ratio Selection
if(~exist('osize', 'var'))
	osize = 0;
end

MIN_AR = 0.625;
MAX_AR = 1.600;

if(osize)
	% "Original" (????px On Longest Dimension)
	URL_SUFFIX = '_o';
	MIN_PIXELS = floor(1024 / MAX_AR); 	% We only want "originals" if they are at least "large" 
										% (1024px) size.
else
	% "Medium" (500px On Longest Dimension)
	URL_SUFFIX = '';
	MIN_PIXELS = floor(500 / MAX_AR);
end

%download images listed in every text file in this directory
search_result_dir = fullfile(search_result_dir);
%directory where you want to download the images
output_dir = fullfile(output_dir);
%the algorithm will create subdirs and subsubdirs in the above dir.

%with the number of images we're expecting we will need two levels of
%subdirectories.  The first level will be the tag, of which there will be
%roughly 100, then the second level will be subdirs of 1000 images.  The
%images will be named such that they can be traced back to their flickr
%source.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fprintf('Reading image metadata from %s\n', search_result_dir);
search_results = dir(fullfile(search_result_dir, '*.txt'));
num_results = length(search_results);
rand('twister', sum(100*clock)); %seed random number generator
search_results = search_results(randperm(num_results)); %randomizing order
% search_results = [search_results; search_results]; %two passes through to make sure
fprintf(' Downloading the images in %d search results\n',num_results)

for i = 1:size(search_results,1)
    current_filename = search_results(i).name;
    current_filename_fh = current_filename(1:end-4); %cutting off .txt extension 
    
    fprintf('\n !!! Checking for lock on %s\n', current_filename)
    
    %The presence of an output directory is a lock
    if ( ~exist(fullfile(output_dir,current_filename_fh), 'file'))
        
        fprintf(' locking %s\n', current_filename)
        %lock the file by creating the output directory
        cmd = ['mkdir ' fullfile(output_dir,current_filename_fh)];
        system(cmd);
        
        % file where the metadata is located
        fid = fopen(fullfile(search_result_dir, current_filename),'r');
        fprintf(' Reading search results in file %s\n', current_filename);

        count=0;
        dircount=0;

        % downloads images, 1000 images per directory
        while 1
          line = fgetl(fid);
          if (~ischar(line))
            break
          end
          
            % photo: 3059572473 0073371b0c 3187
            % owner: 42682565@N00
            % title: ??
            % originalsecret: null
            % originalformat: null
            % o_height: null
            % o_width: null
            % datetaken: 2008-08-14 22:09:25
            % dateupload: 1227665360
            % tags: city sunset sea summer cloud mountain canada beach bike vancouver bay boat ship bc seattleandvancouver
            % license: 0
            % latitude: 49.281468
            % longitude: -123.17871
            % accuracy: 10
            % views: 35
            % interestingness: 3 out of 239
           
          if strncmp(line,'photo:',6)
              
            if mod(count,1000)==0
              dircount = dircount + 1;
              cmd = ['mkdir ' fullfile(output_dir, current_filename_fh, sprintf('%.5d', dircount))];
              system(cmd);
            end
            count=count+1;
              
            first_line = line;
            
            [t,r] = strtok(line);
            [id,r] = strtok(r);
            [secret,r] = strtok(r);
            [server,r] = strtok(r);
            
            line = fgetl(fid); %owner: line
            second_line = line;
            [t,r] = strtok(line);
            [owner,r] = strtok(r);

            %save all the metadata in the comment field of the file
            comment_field = strvcat(first_line, ... %photo id secret server
                                    second_line,... %owner
                                    fgetl(fid), ... %title
                                    fgetl(fid), ... %original_secret            
                                    fgetl(fid), ... %original_format
                                    fgetl(fid), ... %o_height
                                    fgetl(fid), ... %o_width
                                    fgetl(fid), ... %datetaken
                                    fgetl(fid), ... %dateupload
                                    fgetl(fid), ... %tags
                                    fgetl(fid), ... %license
                                    fgetl(fid), ... %latitude
                                    fgetl(fid), ... %longitude
                                    fgetl(fid), ... %accuracy
                                    fgetl(fid), ... %views
                                    fgetl(fid));    %interingness (string)
            
            [t, r] = strtok(comment_field(4,:));
            [originalsecret, r] = strtok(r);
            if osize && ~strcmp(originalsecret,'null')
            	secret = originalsecret;
            end
            
			[t, r] = strtok(comment_field(5,:));
			[originalformat, r] = strtok(r);
            if ~osize || strcmp(originalformat,'null')
            	FORMAT = '.jpg';
            else
            	FORMAT = ['.' originalformat];
            end
            
            url = ['http://static.flickr.com/' server '/' id '_' secret URL_SUFFIX FORMAT];
            TEMP_FILE = [id '_' secret '_' server '_' owner FORMAT];
            
            fprintf('\n   current_image :  %s\n', TEMP_FILE)
            
            %download the file to a temporary, local location
            %before saving it in the full path in order to minimize network
            %traffic using the /tmp/ space on any machine.
            
            %we want the file name to identify the image still.  not just be numbered.
            %use the -O [output file name] option
            %-t specifies number of retries
            %-T specifies all timeouts, in seconds.  if it times out does it retry?
            TEMP_FILENAME = fullfile(TEMP_DIR, TEMP_FILE);
            cmd = [WGET_PATH ' -t 3 -T 5 --quiet ' url ' -O ' TEMP_FILENAME];
               
            try
                system(cmd);
            catch
                lasterr
                fprintf('XX!! Error with wget.\n');
            end

            %we need to check if we got a small error .gif back, in which case
            %we'll want to try for the original image.
            current_file_info = dir(TEMP_FILENAME);
            current_file_size = current_file_info.bytes;    

            if(current_file_size < 5000) %if the file is less than 5k, or we got an error .gif instead
               fprintf('X  error downloading default version\n');
            else
                % load the image, resize it, remove border, save it.
                try
                	if strcmp(FORMAT,'.gif')
                		% "Decode" indexed color
                		[current_image, current_map] = imread(TEMP_FILENAME);
                		current_image = reshape(current_map(current_image(:)+1), size(current_image,1), ...
                			size(current_image,2)) .* 255;
                	else
                    	current_image = imread(TEMP_FILENAME);
                	end
                catch
                    lasterr
                    fprintf('XX!! error loading temporary file, which should have been valid\n')
                end
                
                aspect_ratio = size(current_image,2) / size(current_image,1); %width by height
                min_dim_pixels = min( size(current_image,1) , size(current_image,2) );

                if(aspect_ratio <= MAX_AR && ...  
                   aspect_ratio >= MIN_AR && ... 
                   min_dim_pixels >= MIN_PIXELS)

                    current_image = double(current_image) /255;
                    [trimmed_image] = remove_frame(current_image); %delete border
                    
                    aspect_ratio = size(trimmed_image,2) / size(trimmed_image,1); %width by height
                    min_dim_pixels = min( size(trimmed_image,1) , size(trimmed_image,2) );
                    
                    if(aspect_ratio <= MAX_AR && ...  
                       aspect_ratio >= MIN_AR && ... 
                       min_dim_pixels >= MIN_PIXELS )
                    
                        output_filename = fullfile(output_dir, current_filename_fh, sprintf('%.5d', dircount), ...
							[id '_' secret '_' server '_' owner '.jpg']);

                        %lets save all the info we'll need in the comment
                        %section of the file.  We can retrieve this later
                        %with imfinfo()
                        try
                            imwrite(trimmed_image, output_filename, 'JPG', 'Quality', 100, 'Comment', comment_field);

							if osize                            
                            	cmd = [EXIFTOOL_PATH ' -tagsFromFile ' TEMP_FILENAME ' -x Comment ' output_filename];
			                	system(cmd);
			                	delete([output_filename '_original']);
                            end
                            
                            fprintf('!! Successfully wrote %s\n', output_filename)
                        catch
                            lasterr
                            fprintf('XX!! error writing final image\n')
                        end
                    else
                        fprintf('XX!! Too many pixels removed by trimming\n')
                    end

                else
                    %print out the correct failure cases
                    if(min_dim_pixels < MIN_PIXELS)
                        fprintf('XX Image is too small (%d pixels).  \n', min_dim_pixels)
                    end

                    if(aspect_ratio < MIN_AR || aspect_ratio > MAX_AR)
                        fprintf('XX Aspect ratio is bad (%f).\n', aspect_ratio)
                    end
                end
            end

            % delete the temporary file
            try
                delete(TEMP_FILENAME);
            catch
                lasterr
                fprintf('XX!! failed deleting the temporary file\n')
            end
            
            % pause inserted here so as not to piss flickr off...
            % maybe you can take it out?  probably, because the image
            % processing was slow enough.
            pause(2);
          end
        end
        
        fclose(fid);
        
    else
        fprintf(' file is locked / already downloaded, skipping\n')
    end
end %end of all files
