% run_decade_svm_experiment - Given feature descriptors (computed using
%	create_desc_db) for the "Historical Color Image Database" (available from
%	http://graphics.cs.cmu.edu/projects/historicalColor/), reproduces the
%	linear support vector machine classification experiments described in
%	Section 5.2 of "Dating Historical Color Images".
%
%	It is strongly recommended that you obtain and use "GPU-accelerated LIBSVM"
%	[Athanasopoulos et. al., 2011] (http://mklab.iti.gr/project/GPU-LIBSVM),
%	adjusting the settings 'SVM_TRAIN_NAME' and 'SVM_TRAIN_OFFSET' as 
%	appropriate.
%
%	Options:
%		'DESCR_PATH' - The fully-qualified pathname of the directory containing
%			feature descriptors computed for the "Historical Color Image 
%			Database" (see create_desc_db.m).
%		'f' - The number of the image feature to be used in the experiment. 
%			Currently supported values are:
%				1 - Color Co-occurrence Histogram
%				2 - Conditional Probability of Saturation Given Hue
%				3 - Hue Histogram
%				4 - Gist Descriptor
%				5 - Tiny Image
%				6 - L*a*b* Color Histogram
%				7 - All
%		'EXTERNAL' - "Train Classifier For External Data" (Optional, Boolean)
%				0/False [Default]: Perform the classification experiment as
%					described in the publication (i.e. 10 iterations of 
%					random 215/50 training/testing sets taken from the data 
%					set).
%				1/True: Train a classifier using all data from the data set
%					(e.g. for use with external images). Testing will not be
%					performed, and the LIBSVM model files needed for external
%					use will be left on disk (rather than being deleted).
%
function run_decade_svm_experiment(DESCR_PATH, f, EXTERNAL);
	if(nargin < 3)
		EXTERNAL = 0;
	end

	% -- Settings --
	LIBSVM_PATH = '../third_party/libsvm-3.12';
	SVM_TRAIN_NAME = 'svm-train'; % 'svm-train-gpu'
	SVM_TRAIN_OFFSET = 28; % 28 for Version 3.12, 20 for Version 3.0
							%	(i.e. GPU-accelerated LIBSVM)

	if ~(EXTERNAL)
		% Experiment parameters taken from the publication
		TEST_NUM = 50; % Number of images per class to be used for testing.
		NUM_TEST_ROUNDS = 10; 	% Number of random training/testing splits over
								% which accuracy will be averaged.
	else
		% Train SVMs for external use using all data (i.e. 0 test images).
		TEST_NUM = 0;
		NUM_TEST_ROUNDS = 1;
	end
	% -- End Settings --

	% Feature Dimensions
	DIMS_CO_OCCUR = 3 * (32 ^ 2); % "Color Co-occurrence Histogram"
	DIMS_PSH = 512; % "Conditional Probability of Saturation Given Hue"
	DIMS_HUE = 128; % "Hue Histogram"
	DIMS_GIST = 6 * 4 * 5 * 5;	% "Gist Descriptor"
	DIMS_TINY = 3 * (32 ^ 2); % "Tiny Image"
	DIMS_CIELAB = 4 * 14 * 14; % "L*a*b* Color Histogram"
	DIMS_ARRAY = [DIMS_CO_OCCUR DIMS_PSH DIMS_HUE DIMS_GIST DIMS_TINY ...
		DIMS_CIELAB];
	DIMS_ARRAY = [DIMS_ARRAY sum(DIMS_ARRAY)];
	FEATURE_LABELS = {'Color Co-occurrence Histogram' 'P(Saturation|Hue)' ...
		'Hue Histogram' 'Gist Descriptor' 'Tiny Image' ...
		'L*a*b* Color Histogram' 'All'};

	addpath('../util');
	addpath(fullfile(LIBSVM_PATH,'matlab'));

	display('Enumerating descriptors for each decade...');
	[CF, CS] = initialize_clusters(DESCR_PATH);

	% -- Testing Loop --
	crossval_perfs = zeros(NUM_TEST_ROUNDS,1);
	test_perfs = zeros(NUM_TEST_ROUNDS,1);

	r = 1;
	while(r <= NUM_TEST_ROUNDS)
		display(sprintf('Testing Round: %u', r));

		% Perform stratified random sampling for training/testing split
		display('    Splitting training/testing sets...');
		[CF_TRAIN, CF_TEST, CS_TRAIN, CS_TEST] = split_clusters(CF, CS, 2, ...
			TEST_NUM);

		% Extract data for the specified feature from the training & testing 
		%	sets and label it with classes by decade.
		display(sprintf('    Extracting feature %u (%s) from data sets...', ...
			f, FEATURE_LABELS{f}));
		DIMS = DIMS_ARRAY(f);
		training_data = zeros(0, DIMS);
		training_labels = zeros(0, 1);
		testing_data = zeros(0, DIMS);
		testing_labels = zeros(0, 1);
		for i = 1:numel(CS)
			feature_array = extract_feature_array_from_cluster(CF_TRAIN, ...
				CS_TRAIN, i, f);
			training_data = vertcat(training_data, feature_array);
			training_labels = vertcat(training_labels, i*ones(CS_TRAIN(i), 1));

			feature_array = extract_feature_array_from_cluster(CF_TEST, ...
				CS_TEST, i, f);
			testing_data = vertcat(testing_data, feature_array);
			testing_labels = vertcat(testing_labels, i*ones(CS_TEST(i), 1));
		end

		% Write feature data to disk (LIBSVM)
		display('    Writing feature data to disk...');
		libsvmwrite('training', training_labels, sparse(training_data));
		libsvmwrite('testing', testing_labels, sparse(testing_data));

		% Scale feature data (LIBSVM)
		display('    Scaling feature data...');
		command = sprintf('%s -l 0 -u 1 -s range training > training.scale', ...
			fullfile(LIBSVM_PATH,'svm-scale'));
		[status, result] = system_with_dyld_reset_mac(command);
		command = sprintf('%s -r range testing > testing.scale', ...
			fullfile(LIBSVM_PATH,'svm-scale'));
		[status, result] = system_with_dyld_reset_mac(command);
		delete('training','testing');
		if ~(EXTERNAL)
			% 'range' is required for external usage
			delete('range');
		end
		movefile('training.scale','training');
		movefile('testing.scale','testing');

		% Search for optimal parameters by maximizing performance on the 
		%	training set in ten-fold cross-validation (LIBSVM)
		display('    Searching for optimal parameters...');
		Cs = 2 .^ (-5:2:15);
		perfs = zeros(numel(Cs),1);
		parfor i = 1:numel(Cs)
			command = sprintf('%s -t 0 -c %f -q -v 10 training', ...
				fullfile(LIBSVM_PATH, SVM_TRAIN_NAME), Cs(i));
			[status, result] = system_with_dyld_reset_mac(command);

			perfs(i) = str2num(result(SVM_TRAIN_OFFSET:end-2));
		end
		[Y, IDX] = max(perfs);
		display(sprintf('        Cross-Validation Accuracy: %f (C = %f)', ...
			Y, Cs(IDX)));

		% Train SVMs using the chosen parameters (LIBSVM)
		display('    Training SVMs using chosen parameters...');
		command = sprintf('%s -t 0 -c %f -q training', ...
			fullfile(LIBSVM_PATH, SVM_TRAIN_NAME), Cs(IDX));
		[status, result] = system_with_dyld_reset_mac(command);

		if (EXTERNAL)
			% No testing in external mode...
			r = r + 1;
			continue;
		end
		
		% Test the accuracy of the SVMs on the testing data
		display('    Testing SVMs...');
		command = sprintf('%s testing training.model testing.output', ...
			fullfile(LIBSVM_PATH,'svm-predict'));
		[status, result] = system_with_dyld_reset_mac(command);
	
		perf = str2num(result(12:15));
		if isempty(perf)
			display('        Error interfacing with svm-predict. Will retry.');		
		else
			display(sprintf('        Testing Accuracy: %f', perf));
			crossval_perfs(r) = Y;
			test_perfs(r) = perf;
			r = r + 1;
		end
	
		% Clean up...
		delete('training','testing','training.model','testing.output');
		display(sprintf('Finished Testing Round %u.', r));
	end
	% -- End Testing Loop --
	display('Done.');

	if ~(EXTERNAL)
		display('----TEST REPORT----');
		display(sprintf('Test Runs: %u', r-1));
		display(sprintf('Feature: %s.', FEATURE_LABELS{f})); 
		display(sprintf('Mean Training Cross-Validation Accuracy: %f', ...
			mean(crossval_perfs)));
		display(sprintf('Mean Testing Accuracy: %f', mean(test_perfs)));
	else
		delete('training','testing');
		display('----EXTERNAL MODE----');
		display('LIBSVM Model Files Generated: range, training.model');	
	end
end
