function [IR, CR, CG, CB] = EstimateInverseResponseFromNormalizedJointHistogram(JHR, JHG, JHB)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function has been modified from its originally published version. 
% This modified version is part of:
%
% Palermo, F., Hays, J., Efros, A.A.: Dating Historical Color Images. In:
% Fitzgibbon, A., Lazebnik, S., Sato, Y., Schmid, C. (eds.) ECCV 2012, Part VI.
% LNCS, vol. 7577, pp. 499-512. Springer, Heidelberg (2012)
%
% Specifically, the modifications include:
%	1. Joint histograms JHR, JHB, and JHG are now parameters to the function
%	(rather than variables loaded from a hard-coded MAT file).
%	2. Polynomial coefficients CR, CG, and CB (i.e. coeffsR, coeffsG, and 
%	coeffsB) are now returned to the calling function.
%	3. Built-in visualization of the inverse response curves and certain
%	status outputs have been disabled.
%
% Please do not contact the original authors regarding issues with this 
% modified version of their work.
%
% The original documentation of this function is reproduced below.
% ------------------------------------------------------------------------------
% This function estimates the inverse response function for a camera model 
% using the prior on the joint histogram of irradiances as described in 
% "Priors for Large Photo Collections and What they Reveal about Cameras",
% Sujit Kuthirummal, Aseem Agarwala, Dan B Goldman, and Shree K. Nayar,
% European Conference on Computer Vision (ECCV), 2008.
% 
% The joint histograms are computed from a 31x31 block of pixels in the
% center of images captured at the extreme lens setting -- smallest
% focal length and largest f-number. 
% 
% Author: Sujit Kuthirummal (sujit@cs.columbia.edu)
% Modified last on Sep 18, 2008.
% (C) 2008 Adobe Systems Incorporated. All rights reserved.  
% Provided by Adobe without warranty on an AS-IS basis.  
% Non-commercial use only or as separately expressly permitted by Adobe.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

load PriorJointHistogram_S1IS.mat % Load priors for the joint histograms, computed from S1IS camera model
                                  % Loads variables targetJHR, targetJHG, and targetJHB

% JHR, JHG, and JHB are now parameters.                                 
%load JointHistSonyW1F7.9N5.6.mat  % Load Joint Histograms of intensities -- 256x256 matrices,
                                  % one for each color channel
                                  % Loads variables JHR JHG JHB
                                  
polyDegree = 5;                   % Use 5th degree polynomial for the inverse response

minimumFrequency = 10;            % A bin in the joint histogram should have atleast these many votes to 
                                  % be useful

numEndPointsToIgnore = 5;         % Ignoring 5 bins on all sides of the joint histogram to ignore the 
                                  % effects of saturation and under-exposure in the optimization
                                          
                                  
% Set initial guess: Mean inverse response curve from the EMoR database.
switch polyDegree
    case 4
        initialGuess = [1.54337547133901         -1.92089801114522          1.12119816599198];
    case 5
        initialGuess = [3.65465210173037          -7.5932547829868          6.19857221022627         -1.92137691173949];
    case 6
        initialGuess = [5.31718901960796         -12.2969149570936          10.5299982841576   -3.46198882602259         0.490823991993729 ];
end

% Estimate inverse response
[IR, CR, CG, CB] = EstimateInverseResponse(JHR, JHG, JHB, targetJHR, targetJHG, targetJHB, initialGuess, ...
                                                    minimumFrequency, numEndPointsToIgnore);
                                                
% Built-in visualization of the inverse response curves has been disabled.
%intensities = linspace(0,1,256);
%figure
%plot(intensities, inverseResponsesEstimated.R,'r'); hold on
%plot(intensities, inverseResponsesEstimated.G,'g'); 
%plot(intensities, inverseResponsesEstimated.B,'b'); 
%xlabel('Intensities');
%ylabel('Irradiances');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [inverseResponses, coeffsR, coeffsG, coeffsB] = EstimateInverseResponse(JHR, JHG, JHB, targetJHR, targetJHG, targetJHB, ...
                                                    initialGuess, minimumFrequency, numEndPointsToIgnore)
                         
% Create a mask that ignores numEndPointsToIgnore bins on all sides of the joint histogram to ignore the 
% effects of saturation and under-exposure in the optimization                                  
validMask = FindValidMask(numEndPointsToIgnore);


options = optimset('Display','off','MaxFunEvals',90000);

xx = linspace(0,1,256); % Intensities

%%%% KL Divergence Optimization %%%%%%
%fprintf(1,'********************** Red ... ****************\n');
coeffsR = fminsearch(@OptimizeThisKL,initialGuess,options,JHR, targetJHR, validMask, minimumFrequency);
coeffsR(end+1) = 1-sum(coeffsR);
coeffsR(end+1) = 0;
inverseResponses.R = polyval(coeffsR,xx);


%fprintf(1,'********************** Green ... ****************\n');
coeffsG = fminsearch(@OptimizeThisKL,initialGuess,options,JHG, targetJHG, validMask, minimumFrequency);
coeffsG(end+1) = 1-sum(coeffsG);
coeffsG(end+1) = 0;
inverseResponses.G = polyval(coeffsG,xx);


%fprintf(1,'********************** Blue ... ****************\n');
coeffsB = fminsearch(@OptimizeThisKL,initialGuess,options,JHB, targetJHB, validMask, minimumFrequency);
coeffsB(end+1) = 1-sum(coeffsB);
coeffsB(end+1) = 0;
inverseResponses.B = polyval(coeffsB,xx);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [errorVal] = OptimizeThisKL(currentEstimate, JH, targetJH, validMask, minimumFrequency)
% This function is minimized by fminsearch.
% Arguments:
%  currentEstimate : Coefficients of the polynomial that represents the
%                    current estimate of the inverse response function.
%  JH              : Joint histogram of intensities of one color channel
%                    of the camera model whose response is being estimated.
%  targetJH        : Prior joint histogram of irradiances for that color
%                    channel
%  validMask       : Mask that indicates which bins of the joint histogram
%                    will be used for estimating the inverse response
%                    (Bins on the sides of the histogram are ignored to 
%                    avoid the effects of saturation and under-exposure)
%  minimumFrequency: A bin in the joint histogram should have atleast these
%                    many votes to be useful

currentEstimate(end+1) = 1-sum(currentEstimate);
currentEstimate(end+1) = 0;
xx = linspace(0,1,256)';
g = polyval(currentEstimate,xx)'*255;

% Enforce monotonicity
firstDerivative = diff(g);
if min(firstDerivative) <= 0
    errorVal = 1e100; % Current estimate is not monotonically increasing
    return;
    % This also enforces that min(g) >=0  && max(g) <= 255 
end

% Get the bin sizes
tmp = (g(2:end)-g(1:end-1))/2;
binSizes = [2*tmp(1), tmp(1:end-1)+tmp(2:end), 2*tmp(end)];

binAreas = binSizes'*binSizes;

% Normalize joint histogram by bin areas in irradiance space
JH = JH./binAreas;

% Sample the joint histogram so that we have uniform sampling of radiance
[x,y] = meshgrid(g,g);
[u,v] = meshgrid(0:255,0:255);
JH = interp2(x,y,JH,u,v,'bilinear');

% A bin in the joint histogram should have atleast minimumFrequency votes 
% to be useful                                
validMask(JH       < minimumFrequency) = 0; 
validMask(targetJH < minimumFrequency) = 0;

JH = JH(validMask);
targetJH = targetJH(validMask);

% Normalize so that the probabilities sum to 1 
JH = JH/sum(JH);
targetJH = targetJH/sum(targetJH);

% Compute the symmetric KL Divergence
KL1 =   targetJH .* ( log(targetJH) - log(JH) );
KL2 =   JH .* ( log(JH) - log(targetJH) );

errorVal = sum(KL1)+sum(KL2);
% fprintf(1,'%g\n',errorVal);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function validMask = FindValidMask(numEndPointsToIgnore)
% This function creates a mask that is 1 everywhere except for a
% numEndPointsToIgnore size border.

validMask = true(256,256);
validMask(1:numEndPointsToIgnore,:) = 0;
validMask(end-numEndPointsToIgnore+1:end,:) = 0;
validMask(:,1:numEndPointsToIgnore) = 0;
validMask(:,end-numEndPointsToIgnore+1:end) = 0;
