% extract_feature_array_from_cluster - Given cluster data structures CF and CS 
%	(from initialize_clusters or split_clusters), extracts feature data for
%	feature 'f' from the image descriptors of cluster 'i', returning the data
%	in 'feature_array'.
%
%	Options:
%		'i' (REQUIRED): The number of the cluster from which to extract data.
%			1 <= i <= numel(CS)
%		'f' (REQUIRED): The number of the image feature to be extracted:
%			Currently supported values are:
%				1 - Color Co-occurrence Histogram
%				2 - Conditional Probability of Saturation Given Hue
%				3 - Hue Histogram
%				4 - Gist Descriptor
%				5 - Tiny Image
%				6 - L*a*b* Color Histogram
%				7 - All
%
function [feature_array] = extract_feature_array_from_cluster(CF, CS, i, f);
	% Feature Dimensions
	DIMS_CO_OCCUR = 3 * (32 ^ 2); % "Color Co-occurrence Histogram"
	DIMS_PSH = 512; % "Conditional Probability of Saturation Given Hue"
	DIMS_HUE = 128; % "Hue Histogram"
	DIMS_GIST = 6 * 4 * 5 * 5;	% "Gist Descriptor"
	DIMS_TINY = 3 * (32 ^ 2); % "Tiny Image"
	DIMS_CIELAB = 4 * 14 * 14; % "L*a*b* Color Histogram"
	DIMS_ARRAY = [DIMS_CO_OCCUR DIMS_PSH DIMS_HUE DIMS_GIST DIMS_TINY ...
		DIMS_CIELAB];
	DIMS_ARRAY = [DIMS_ARRAY sum(DIMS_ARRAY)];
	DIMS = DIMS_ARRAY(f);

	feature_array = zeros(CS(i), DIMS);
	for i2 = 1:CS(i)
		desc = load(CF{i,i2});
		
		switch (f)
			case 1
				% "Color Co-occurrence Histogram"
				feature = desc.co_occur_hist;
			case 2
				% "Conditional Probability of Saturation Given Hue"
				feature = desc.P_S_Given_H;
			case 3
				% "Hue Histogram"
				feature = desc.hue_hist;		
			case 4
				% "Gist Descriptor"
				feature = desc.gist_image;
			case 5
				% "Tiny Image"
				feature = desc.tiny_image;
			case 6
				% "L*a*b* Color Histogram"
				feature = desc.CIELAB_hist;
			case 7
				% "All"
				feature = [	desc.co_occur_hist(:); ... 
							desc.P_S_Given_H(:); ...
							desc.hue_hist(:); ...
							desc.gist_image(:); ...
							desc.tiny_image(:); ...
							desc.CIELAB_hist(:)];
		end
		
		feature_array(i2,:) = reshape(feature, [1 DIMS]); 
	end
end
