% initialize_clusters - Enumerates the image descriptors in a folder
%	hierarchy, then creates an initial clustering of the descriptors and
%	returns data structures describing that clustering. The data
%	structures are:
%		
%		CS: An array of size n where CS(j) contains the number of
%			descriptors in cluster j (of n total clusters).
%		CF: A cell array of size (n,max(CS)) where CF{j,i} contains the
%			full path and filename of the i-th descriptor file in cluster
%			j. Clusters may be of differing sizes; entries CF{j,i} where
%			i > CS(j) are not guaranteed to contain valid data.
%
%	Options:
%		'in_path' (REQUIRED): A directory containing the image descriptors 
%			to enumerate and cluster. This must be a fully qualified
%			pathname of a two-level descriptor hierarchy (i.e. folder
%			-> folder(s) -> descriptor(s) ).
%		'mode' (OPTIONAL): The type of grouping to use when constructing
%			the initial clusters. Valid modes are:
%				
%				1 (DEFAULT) - Treat each of the subdirectories of in_path
%					as a cluster containing all descriptors from that
%					directory.
%				2 - Treat each image descriptor under in_path as an
%					individual cluster containing only that single
%					descriptor.
function [CF, CS] = initialize_clusters(in_path, mode);
	% -- Settings --
	CLUSTER_PREALLOC = 300;
	% -- End Settings --
	
	if ~exist('mode','var')
		mode = 1;
	end
	
	sub_dirs = real_dirs(in_path);
	num_sub_dirs = numel(sub_dirs);
		
	CF = cell(num_sub_dirs, CLUSTER_PREALLOC);
	CS = zeros(num_sub_dirs, 1);
	
	% Initialize classes/clusters by subdirectory
	j = 0;
	for i = 1:num_sub_dirs
		frame_descs = dir(fullfile(in_path, sub_dirs{i},'*.mat'));
		
		for i2 = 1:numel(frame_descs)
			j = j + 1;
			
			switch (mode)
				case 1
					CS(i) = CS(i) + 1;
					CF{i, CS(i)} = ...
						fullfile(in_path, sub_dirs{i}, frame_descs(i2).name);
				case 2
					CS(j) = 1;
					CF{j, 1} = ...
						fullfile(in_path, sub_dirs{i}, frame_descs(i2).name);
			end
		end
	end
end
