% split_clusters - Given cluster data structures CF and CS (created by
%	initialize_clusters), splits each cluster via stratified random sampling
%	into two sections returned in the new data structures (CF_A,CS_A) and 
%	(CF_B,CS_B) respectively. The parameter 'split_amt' controls the number of
%	randomly selected items assigned to each 'B' cluster; all remaining items
%	from the input are assigned to the 'A' clusters. The parameter 'split_mode' 
%	controls whether 'split_amt' should be interpreted as a fractional size of
%	the input or as a specific number of items.
%
%	Options:
%		'split_mode' (REQUIRED):
%			1 - "Fractional Split": 'split_amt' will be treated as a decimal
%				fraction of the size of the input clusters; i.e. each 'B'
%				cluster will contain floor([size of input cluster] * split_amt)
%				items.
%			2 - "Fixed Split": 'split_amt' will be treated as a specific
%				number of items; i.e. each 'B' cluster will contain split_amt
%				items.
%		'split_amt' (REQUIRED): The desired size of each 'B' cluster, 
%			interpreted according to 'split_mode' (see above).
function [CF_A, CF_B, CS_A, CS_B] = split_clusters(CF, CS, split_mode, ...
	split_amt);
	
	numClusters = numel(CS);
	
	% Determine size of 'B' clusters based on 'split_mode' & 'split_amt'
	switch (split_mode)
		case 1
			% "Fractional Split"
			CS_B = floor(CS * split_amt);
		case 2
			% "Fixed Split"
			CS_B = split_amt * ones(numClusters, 1);
	end
	
	% All remaining items will be assigned to the 'A' clusters
	CS_A = CS - CS_B;
	CF_B = cell(numClusters, max(CS_B));
	CF_A = cell(numClusters, max(CS_A));
	
	% Stratified random sampling: each cluster is randomly split individually
	for i = 1:numClusters
		% Randomly select 'B' items; all items remaining are 'A' items
		rnd_idxs = randperm(CS(i));
		rnd_idxs_B = rnd_idxs(1:CS_B(i));
		rnd_idxs_A = setdiff(rnd_idxs, rnd_idxs_B);
		
		% Copy selected filenames from CF into CF_A and CF_B.
		for i2 = 1:numel(rnd_idxs_A)
			CF_A{i,i2} = CF{i, rnd_idxs_A(i2)};
		end
		
		for i2 = 1:numel(rnd_idxs_B)
			CF_B{i,i2} = CF{i, rnd_idxs_B(i2)};
		end
	end
end
